@@ -129,6 +129,8 @@ typedef struct {
 	STRAR * g_sourcepath_listM; /* used by swcopy */
 	int g_no_extractM; /* used by swcopy */
 	int g_do_show_linesM; /* used by swverify */
+	char * g_catalog_info_nameM; /* used by swverify */
+	char * g_system_info_nameM; /* used by swverify */
 } GB;
 
 #define G_opta (G->optaM)
