@@ -56,7 +56,7 @@
 #define PROGNAME                "swverify"
 #define SW_UTILNAME             PROGNAME
 #define SWPROG_VERSION          SWVERIFY_VERSION
-#define WOPT_LAST		215   /* Highest option number */
+#define WOPT_LAST		217   /* Highest option number */
 
 static char progName[] = PROGNAME;
 static char * CHARTRUE = "True";
@@ -197,7 +197,9 @@ fprintf(fp, "%s",
 "      --check-mtime=true|false\n"
 "      --check-scripts=true|false    Not implemented\n"
 "      --check-volatile=true|false\n"
-"      --show-info     show the data that is being compared\n"
+"      --show-info    show the matching comparison info to stdout\n"
+"      --catalog-info=NAME  write comparison info to filename NAME\n"
+"      --system-info=NAME  write comparison info to filename NAME\n"
 "\n"
 "  Verification Options for verifying a directory:\n"
 "\n"
@@ -685,6 +687,8 @@ main(int argc, char **argv)
 		{"check-permissions", 1, 0, 213},
 		{"check-scripts", 1, 0, 214},
 		{"show-info", 0, 0, 215},
+		{"catalog-info", 1, 0, 216},
+		{"system-info", 1, 0, 217},
 		{0, 0, 0, 0}
              };
 
@@ -931,7 +935,7 @@ main(int argc, char **argv)
 	wopt_allow_ambig 		= CHARFALSE;
 	wopt_noscripts 			= CHARFALSE;
 	wopt_enforce_all_signatures	= CHARFALSE;
-	wopt_sig_level			= "0";
+	wopt_sig_level			= "1"; /* default sig-level */
 
 	wopt_show_options = NULL;
 	wopt_show_options_files = NULL;
@@ -1415,6 +1419,12 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 		case 215:
 			G->g_do_show_linesM = 1;
 			break;
+		case 216:
+			G->g_catalog_info_nameM = strdup(optarg);
+			break;
+		case 217:
+			G->g_system_info_nameM = strdup(optarg);
+			break;
 		default:
 			sw_e_msg(G, "error processing implementation extension option : %s\n",
 				cplob_get_list(w_arglist)[w_option_index]);
