@@ -39,6 +39,12 @@ volatile int global_lastsig;
 #define SWINSTALL_VOLATILE_SUFFIX	".swbisnew"
 #define SWREMOVE_VOLATILE_SUFFIX	".swbisold"
 
+#define SWC_SHELL_KEY_SH	"sh"
+#define SWC_SHELL_KEY_KSH	"ksh"
+#define SWC_SHELL_KEY_BASH	"bash"
+#define SWC_SHELL_KEY_POSIX	"posix"
+#define SWC_SHELL_KEY_DETECT	"detect"
+
 #define PAX_READ_COMMANDS_LEN	10
 #define PAX_WRITE_COMMANDS_LEN	10
 #define SW_UTILNAME		"swutility"
@@ -106,8 +112,10 @@ volatile int global_lastsig;
 #define SWBIS_TS_make_locked_session	"Lock Session"
 #define SWBIS_TS_released_locked_session  "UnLock Session"
 #define SWBIS_TS_check_loop  		"Check Loop"
+#define SWBIS_TS_continue		"Programmed Continue"
 #define SWBIS_TS_report_status 		"Report Status"
 #define SWBIS_TS_remove_catalog_entry	"Remove catalog entry"
+#define SWBIS_TS_restore_catalog_entry	"Restore catalog entry"
 #define SWBIS_TS_remove_files		"Remove files"
 #define SWBIS_TS_retrieve_files_archive "Retrive files archive"
 #define SWBIS_TS_post_verify		"Run post Verify"
