@@ -2447,7 +2447,7 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 						) {
 							/* don't allow ambiguaous selections (e.g. wildcards) unless
 							   specifically allowed via the --allow-ambig option */
-							E_DEBUG();
+							E_DEBUG("");
 							swicat_sr_form_swspec(sr, btmp2);
 							sw_e_msg(G, "SW_SELECTION_NOT_FOUND_AMBIG: %s: status=1\n",
 								strob_str(btmp2));
@@ -2473,7 +2473,7 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 						E_DEBUG2("swc_looper_sr_payload returned %d", ret);
 						if (rp_status != 0) {
 							/* error, but continuation possible */
-							E_DEBUG();
+							E_DEBUG("");
 							if (rp_status != SWVERIFY_RP_STATUS_NO_INTEGRITY)
 								sw_e_msg(G, "SW_INTERNAL_ERROR: %s:%d rp_status=%d\n",
 									__FILE__, __LINE__, rp_status);
@@ -2482,7 +2482,7 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 						E_DEBUG2("signal_flag=%d", G->g_signal_flag);
 						if (ret < 0) {
 							/* error that is incompatible with continuing */
-							E_DEBUG();
+							E_DEBUG("");
 							raise(SIGINT);
 							f_retval++;
 							break;
@@ -2556,7 +2556,7 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 				}
 			} else if (swpl_test_pgm_mode(wopt_pgm_mode, SWLIST_PMODE_INDEX) == 0) {
 				E_DEBUG("");
-				sw_e_msg(G, "INDEX file format list mode unsupported\n");
+				sw_e_msg(G, "software definition file format list mode unsupported\n");
 				swicol_set_master_alarm(swicol);
 				swicol_set_task_idstring(swicol, SWBIS_TS_Get_iscs_listing);
 				abort_script(swicol, G, target_fdar[1]);
