@@ -66,12 +66,20 @@ print_lines(char * name, char * pref, STRAR * ar)
 	int i;
 	i = 0;
 
-	fd = open(name, O_RDWR|O_CREAT|O_TRUNC, 0644);	 
+	if (strcmp(name, "-") == 0) {
+		fd = STDOUT_FILENO;
+	} else {
+		fd = open(name, O_RDWR|O_CREAT|O_TRUNC, 0644);	 
+	}
+
+	if (fd < 0) {
+		fprintf(stderr, "%s: info dump failed: %s\n", swlib_utilname_get(), strerror(errno));
+		return;
+	}
 	s = strar_get(ar, i++);
 	while (s) {
 		write(fd, s, strlen(s));
 		write(fd, "\n", 1);
-		/* fprintf(stderr, "%s%s\n", pref, s); */
 		s = strar_get(ar, i++);
 	}
 	close(fd);
@@ -258,6 +266,7 @@ get_name_len(char * line)
 		s = strstr(s+1, " " LS_LIST_LINKNAME_MARK);
 	}
 	if (! l) {
+fprintf(stderr, "[%s]\n", line);
 		SWLIB_FATAL("");
 	}
 	E_DEBUG2("returning %d", (int)(l - line));
@@ -517,6 +526,7 @@ compare_fileset(GB * G, char * swspec_string, char * target_path, SWUTS * uts, S
 			ret = xformat_copy_pass_digs(xformat, nullfd, archive_fd, digs);
 		}
 		E_DEBUG("");
+		available_attributes |= LS_LIST_VERBOSE_PREPEND_DOTSLASH; 
 		taru_print_tar_ls_list(tmp, file_hdr, available_attributes);
 		uxfio_write(sys_fd, strob_str(tmp), strob_strlen(tmp));
 	}
@@ -524,18 +534,19 @@ compare_fileset(GB * G, char * swspec_string, char * target_path, SWUTS * uts, S
 	meta_text = uxfio_get_fd_mem(meta_fd, (int *)NULL);
 	sys_text = uxfio_get_fd_mem(sys_fd, (int *)NULL);
 
-	/* 
-	fprintf(stdout, "%s", meta_text);
-	fprintf(stdout, "%s", sys_text);
-	*/
-
 	/* split into lines */
 	meta_str = linize(meta_text);
 	sys_str = linize(sys_text);
+	
 
 	/* sort the lines */
-	strar_qsort(meta_str, swi_fl_compare);
-	strar_qsort(sys_str, swi_fl_compare);
+	strar_qsort(meta_str, strar_qsort_strcmp);
+	strar_qsort(sys_str, strar_qsort_strcmp);
+
+	if (G->g_catalog_info_nameM)
+		print_lines(G->g_catalog_info_nameM, "", meta_str);
+	if (G->g_system_info_nameM)
+		print_lines(G->g_system_info_nameM, "", sys_str);
 
 	/* Now compare these line by line */
 
@@ -547,11 +558,6 @@ compare_fileset(GB * G, char * swspec_string, char * target_path, SWUTS * uts, S
 		*compare_status = ret;
 	}
 
-	/*
-	print_lines("/tmp/meta.txt", "", meta_str);
-	print_lines("/tmp/sys.txt", "", sys_str);
-	*/
-
 	strar_close(meta_str);
 	strar_close(sys_str);
 	uxfio_close(sys_fd);
