@@ -7,7 +7,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,14 +17,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef swexdistribution_i_h
 #define swexdistribution_i_h
 
+extern "C" {
+#include "swuser_config.h"
+}
+
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
+#include <limits.h>
 #include <sys/stat.h>
 #include "swmetadata.h"
 #include "swexstruct_i.h"
@@ -177,7 +183,7 @@ class swExDistribution: public swExCat
 		tmp = strob_open(20);
 		strob_strcpy(tmp, path);
 		swlib_add_trailing_slash(tmp);
-		ret = swExStruct_i::swexstruct_write_dir(strob_str(tmp), getCreateTime(), 
+		ret = swExStruct_i::swexstruct_write_dir(strob_str(tmp), getDirMtime(), 
 			getLeadingDirModeString(),
 			getLeadingDirOwner() /* swdef->find(SWBIS_DISTRIBUTION_OWNER_ATT) */,
 			getLeadingDirGroup() /* swdef->find(SWBIS_DISTRIBUTION_GROUP_ATT) */);
