@@ -1,35 +1,36 @@
 /* swextendeddef.h  -- Process PSF Extended Definitions.
- */
 
-/*
-   Copyright (C) 2003,2004,2005 Jim Lowe
-   All Rights Reserved.
+ Copyright (C) 2003,2004,2005,2007 Jim Lowe
+ All Rights Reserved.
   
-   COPYING TERMS AND CONDITIONS
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-  
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-  
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+ COPYING TERMS AND CONDITIONS:
+ This program is free software; you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation; either version 3, or (at your option)
+ any later version.
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
+ You should have received a copy of the GNU General Public License
+ along with this program; if not, write to the Free Software
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
+*/
 
+#ifndef swextdef_20070211_h
+#define swextdef_20070211_h
 
-#ifndef swextdef_20041023_h
-#define swextdef_20041023_h
-
+extern "C" {
+#include "swuser_config.h"
+}
 
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
 #include <assert.h>
+#include <limits.h>
 #include <unistd.h>
+#include <string.h>
 #include "swuser_config.h"
 #include "swmetadata.h"
 #include "swattributemem.h"
@@ -48,6 +49,9 @@ extern "C" {
 #include "taru.h"
 }
 
+#define SWEXTDEFNEEDDEBUG 1
+#undef SWEXTDEFNEEDDEBUG
+
 #ifdef SWEXTDEFNEEDDEBUG
 #define SWEXTDEF_DEBUG(format) SWBISERROR("SWEXTDEF DEBUG: ", format)
 #define SWEXTDEF_DEBUG2(format, arg) SWBISERROR2("SWEXTDEF DEBUG: ", format, arg)
@@ -101,8 +105,10 @@ class swExtendedDef : private swMetaData
 	int	did_cwd_prefixM;	// True or False,  Did the directory.source value get prefixed with the cwd.
 	int	do_prefix_distribution_targetM;
 	int	do_create_cksumM;
-	int	do_create_md5sumM;
+	int	do_create_md5sumM;	// sha1 and md5
+	int	do_create_sha2M;	// sha512
 	int	non_existing_sourceM;
+	int     verbose_levelM;
 
 	STROB * excludeM;
 	STROB * includeM;
@@ -127,6 +133,7 @@ public: // =====================================================================
 		init_directory();
 		init_file_recursive();
 		set_swvarfs((SWVARFS*)swvarfs);
+		verbose_levelM = ::swlib_get_verbose_level();
 	}
 	
 	swExtendedDef(void) : swMetaData() {
@@ -162,6 +169,13 @@ public: // =====================================================================
 
 	// Public
 	//
+	void
+	set_digests2_creation(int s)
+	{
+		SWEXTDEF_DEBUG("ENTER");
+		do_create_sha2M = s;
+	}
+	
 	void
 	set_digests1_creation(int s)
 	{
@@ -329,14 +343,22 @@ public: // =====================================================================
 				path = "./";
 			}
 			
-			ret=processExplicitFile_i(path, swdef, &st, 1, 1, NULL);
-			assert(swdefFormedM);
-			swdefFormedM->set_no_stat(1);
-			recursive_list_unique_add(psf, swdefFormedM);
-			if (fileset_recursive_startM < 0) {
-				fileset_recursive_startM = psf->swdeffile_get_index_from_pointer(swdefFormedM);
+			ret = processExplicitFile_i(path, swdef, &st, 1, 1, NULL);
+			if (ret == 0) {
+				assert(swdefFormedM);
+				swdefFormedM->set_no_stat(1);
+				recursive_list_unique_add(psf, swdefFormedM);
+				if (fileset_recursive_startM < 0) {
+					fileset_recursive_startM = psf->swdeffile_get_index_from_pointer(swdefFormedM);
+				}
+				set_start_index(psf, swdefFormedM);
+			} else if (ret < 0) {
+				// error
+				fprintf(stderr,"swpackage: error processing file: %s\n", path);
+			} else if (ret > 0) {
+				// Ignore, such as a socket
+				fprintf(stderr,"swpackage: %s: socket ignored\n", path);
 			}
-			set_start_index(psf, swdefFormedM);
 			if ((ret=swvarfs_dirent_err(swvarfs))) {
 				fprintf(stderr,"swpackage: internal error status returned from swvarfs_get_next_dirent().\n");
 				return (ret);
@@ -389,12 +411,17 @@ public: // =====================================================================
 			// Explicit file definition 5.2.13.3
 			//
 			SWEXTDEF_DEBUG("");
-			if ((retval=processExplicitFile(value, swdef))) {
+			retval = processExplicitFile(value, swdef);
+			if (retval < 0) {
 				SWEXTDEF_DEBUG("");
-				fprintf(stderr, "swpackage: error processing: file %s\n",
+				fprintf(stderr, "swpackage: error processing file: %s\n",
 					value /*strob_str(file_pathM)*/);
 				//fprintf(stderr,"Fatal Internal Software Error indicated by return from processExplicitFile().\n");
 				//exit(41);
+			} else if (retval > 0) {
+				// ignore socket
+				fprintf(stderr, "swpackage: %s: socket ignored\n", value /*strob_str(file_pathM)*/);
+				retval = 0;
 			} else {
 				assert(swdefFormedM);
 				//
@@ -539,7 +566,12 @@ public: // =====================================================================
 		int do_no_stat = 1;
 		int ret = processExplicitFile_i(value, parent_swdef, static_cast<struct stat *>(NULL), 1, 0, &do_no_stat);
 		// The NULL st buf causes the stat to be done in stat_source_file_if().
-		if (swdefFormedM) swdefFormedM->set_no_stat(do_no_stat); 
+		if (swdefFormedM && ret == 0)
+			swdefFormedM->set_no_stat(do_no_stat); 
+		if (ret > 0) {
+			// Not an error, the file was ignored
+			;
+		}
 		return ret;
 	}
 
@@ -684,106 +716,6 @@ public: // =====================================================================
 		return 0;
 	}
 
-	// Public
-	//
-	// 5.2.13.5 Exclude Files
-	//
-	int
-	processExcludesOld(swDefinitionFile* psf, char * value) {
-		STROB * psrc;
-		struct stat st;
-		int i = excl_sindexM;
-		swDefinition * swdef;
-		char * source;
-		char * isource;
-		char * misource;
-		int is_dir = 0;
-		int t_dflag = 0;
-		int l_dflag = 0;
-
-		SWEXTDEF_DEBUG2("ENTER %s", value);
-		psrc = strob_open(24);
-		parse_SourcePath(value, &source, &isource);	// isource is a dummy variable in this usage.
-
-		//
-		// Determine if source is a directory.
-		//
-
-		if (swlib_dir_compare(strob_str(directory_sourceM), source, SWC_FC_NOAB) == 0) {
-			//
-			// directory_sourceM and source are the same.
-			//
-			get_QualifiedAbsoluteSourcePath(psrc, directory_sourceM, ""); 
-		} else {
-			get_QualifiedAbsoluteSourcePath(psrc, directory_sourceM, source); 
-		}
-
-		SWEXTDEF_DEBUG2("source is %s", source);
-		SWEXTDEF_DEBUG2("directory_sourceM is %s", strob_str(directory_sourceM));
-		SWEXTDEF_DEBUG2("psrc is %s", strob_str(psrc));
-		if (local_lstat_swvarfs(strob_str(psrc), &st) < 0) {
-			fprintf(stderr,"swpackage: warning: exclude definition source [%s] does not exist.\n", strob_str(psrc));
-			is_dir = 0;
-		} else {
-			if (S_ISDIR(st.st_mode)) {
-				is_dir = 1;
-			}
-		}
-		
-		swlib_toggle_trailing_slashdot("drop", source, &t_dflag);
-		swlib_toggle_leading_dotslash("drop", source, &l_dflag);
-
-		while ((swdef=psf->swdeffile_get_pointer_from_index(i++)) != static_cast<swDefinition*>(NULL)) {
-			SWEXTDEF_DEBUG2("keyword is %s", swdef->get_keyword());
-			if(strcmp("file", swdef->get_keyword())) {
-				//
-				// Sanity check
-				// This really is an error, not a warning.
-				fprintf(stderr,"swpackage: Error: Non file object found in list in processExcludes().\n");
-				strob_close(psrc);
-				return -1;
-			}		
-			isource = swdef->find(SW_A_source);
-			if (isource == static_cast<char *>(NULL)) {
-				fprintf(stderr,"swpackage: error: path attribute not found in processExcludes().\n");
-				strob_close(psrc);
-				return -1;
-			}
-
-			//
-			// Strip off absolute part if directory_sourceM is relative.
-			//
-			if (*strob_str(directory_sourceM) != '/' && excl_qindexM > 0) {
-				misource = isource + ((excl_qindexM > (int)strlen(isource)) ? strlen(isource) : excl_qindexM);
-			} else {
-				misource = isource;
-			}
-			//
-			// Now compare source to misource to determine if to exclude.
-			//
-
-			SWEXTDEF_DEBUG3("comparing for exclusion : [%s] [%s]", misource, source);
-			if (compare_exclude(misource, source, is_dir) == 0) {
-				// Do exclude.
-				psf->swdeffile_list_del(--i);		
-				SWEXTDEF_DEBUG3("compared for exclusion : result=EXCLUDED [%s] [%s]", misource, source);
-				if (!is_dir && pattern_is_wild(source) == 0) {
-					// Only match one time.
-					break;
-				}
-			} else {
-				;
-				SWEXTDEF_DEBUG3("compared for exclusion : result=RETAINED [%s] [%s]", misource, source);
-			}
-		}
-
-		swlib_toggle_trailing_slashdot("restore", source, &t_dflag);
-		swlib_toggle_leading_dotslash("restore", source, &l_dflag);
-
-		strob_close(psrc);
-		return 0;
-	}
-	
 	// Public
 	//
 	int 
@@ -885,6 +817,25 @@ private: // ================================================================
 		i = 0;
 		pat = strar_get(pat_list, i++);
 		while(pat && ret) {
+			//
+			// Check for the case
+			//   where pat=/dir/A/B/C/  and mdef_source=/dir/A/B/C
+			// As an implementation extension call this a match
+			// (fnmatch wil not).   This is used to exlude a single
+			// directory with out excluding the contents of /dir/A/B/C
+			//
+			if (	pat &&
+				strlen(pat) > 1 &&
+				*(pat+strlen(pat)-1) == '/' &&
+				strlen(mdef_source) == strlen(pat)-1 &&
+				strpbrk(pat, "*?") == NULL &&
+				strncmp(pat, mdef_source, strlen(pat)-1) == 0
+			) {
+				// Got a match for the special case to exlude
+				// a directory but not its contents
+				ret = 0;
+				break;
+			}
 			ret = fnmatch(pat, mdef_source, 0);
 			if (ret == 0) break;
 			pat = strar_get(pat_list, i++);
@@ -1020,6 +971,7 @@ private: // ================================================================
 				case cksumE:
 				case md5sumE:
 				case sha1sumE:
+				case sha512sumE:
 				case mtimeE:
 				case ctimeE:
 				case inoE:
@@ -1062,6 +1014,7 @@ private: // ================================================================
 		distribution_targetM = strob_open(10);
 		do_create_cksumM = 0;
 		do_create_md5sumM = 0;
+		do_create_sha2M = 0;
 		did_open_swvarfsM = 0;
 		non_existing_sourceM = 0;
 	}
@@ -1464,23 +1417,30 @@ private: // ================================================================
 		char * s;
 		char * ret;
 		
+		SWEXTDEF_DEBUG3("path=[%s]  prefix=[%s]", path, prefix);
 		s = strstr(path, prefix);
 		if (s == path && strcmp(path, prefix) == 0) {
 			ret = path + strlen(prefix);
+			SWEXTDEF_DEBUG("");
 			//
 			// This a empty string.
 			//
 		} else if (s == path) {
+			SWEXTDEF_DEBUG("");
 			ret = path + strlen(prefix);
 			while( *ret == '/') ret++;
+		} else if (strcmp(prefix, ".") == 0) {
+			SWEXTDEF_DEBUG("");
+			ret = path;
 		} else {
 			// SWBIS_E_FAIL("");
+			SWEXTDEF_DEBUG("");
 			ret = static_cast<char*>(NULL);
 		}
+		SWEXTDEF_DEBUG("");
 		return ret;	
 	}
 
-
 	//
 	// transform `strb' to an absolute path using `path_prefix'.
 	//
@@ -2123,6 +2083,7 @@ private: // ================================================================
 	//	
 	int
 	stat_source_file_if(char *source, char * hl_path, struct stat *sti) {
+		int retval;
 		char * p_owner;
 		char *p_mode;
 		char *p_minor;
@@ -2141,6 +2102,7 @@ private: // ================================================================
 		struct stat *st;
 		SWEXTDEF_DEBUG("");
 	
+		retval = 0;
 		if (sti != static_cast<struct stat*>(NULL)) {
 			SWEXTDEF_DEBUG("");
 			st = sti;
@@ -2216,6 +2178,9 @@ private: // ================================================================
 			if (do_create_md5sumM) {
 				cksumflags |= SWFILE_DO_FILE_DIGESTS;
 			}
+			if (do_create_sha2M) {
+				cksumflags |= SWFILE_DO_FILE_DIGESTS_SHA2;
+			}
 	
 			init_statArray();
 
@@ -2225,8 +2190,11 @@ private: // ================================================================
 				hlif_path = source;
 			}
 
-			if (swmetadata_decode_file_stats(swvarfsM, source, hlif_path, st,
-				statArrayM, statArrayWasSetM, cksumflags) != 0) {
+			retval = swmetadata_decode_file_stats(swvarfsM, source,
+				hlif_path, st, statArrayM,
+				statArrayWasSetM, cksumflags);
+
+			if (retval < 0) {
 				fprintf(stderr, "swpackage: error processing file stats for %s\n", source);
 				return -1;
 			}
@@ -2237,9 +2205,12 @@ private: // ================================================================
 			if (do_create_md5sumM) {
 				copy_file_cell(fileArrayM, fileArrayWasSetM, md5sumE, statArrayM[md5sumE]);
 			}
+			if (do_create_sha2M) {
+				copy_file_cell(fileArrayM, fileArrayWasSetM, sha512sumE, statArrayM[sha512sumE]);
+			}
 			copy_file_cell(fileArrayM, fileArrayWasSetM, sha1sumE, statArrayM[sha1sumE]);
 		}
-		return 0;
+		return retval;
 	}
 	
 	//
@@ -2300,13 +2271,16 @@ private: // ================================================================
 	// definitions if required.
 	//
 	int 
-	generate_ExplicitFileAttributes(char * source, char * hl_path, struct stat *st, int do_stat){
+	generate_ExplicitFileAttributes(char * source, char * hl_path, struct stat *st, int do_stat) {
+		int retval;
+		int ret;
 		int i;
 		mode_t def_umask;
 		mode_t mode;
 		unsigned int ui_def_umask;
 		unsigned int ui_mode;
-
+		
+		retval = 0;
 		//
 		// Section 5.2.13.3
 		// Lines 810-812.  If file_permissions keyword is active ...
@@ -2337,9 +2311,15 @@ private: // ================================================================
 
 		if (do_stat) {
 			SWEXTDEF_DEBUG("");
-			if (stat_source_file_if(source, hl_path, st)) {
+			ret = stat_source_file_if(source, hl_path, st);
+			if (ret < 0) {
 				fprintf(stderr,"swpackage: stat_source_file_if returned error.\n");
 				exit(2); // FIXME
+			} else if (ret > 0) {
+				// ignore socket
+				retval = ret;
+			} else {
+				retval = ret;
 			}
 		}
 
@@ -2445,7 +2425,7 @@ private: // ================================================================
 
 		SWEXTDEF_DEBUG("");
 		*fileArrayM[umaskE]='\0';
-		return 0;
+		return retval;
 	}
 
 	//
@@ -2575,6 +2555,7 @@ private: // ================================================================
 				case cksumE:
 				case md5sumE:
 				case sha1sumE:
+				case sha512sumE:
 				case mtimeE:
 				case ctimeE:
 				case inoE:
@@ -2640,6 +2621,7 @@ private: // ================================================================
 	processExplicitFile_i(char * value, swDefinition * parent_swdef, struct stat *st,
 				int check_one_active, int recursive_context, int * do_no_stat)
 	{
+		int ret;
 		swDefinition * swdef;
 		char * source_path;
 		char * source;
@@ -2647,6 +2629,7 @@ private: // ================================================================
 		char * o_source;
 		char * o_path;
 
+
 		SWEXTDEF_DEBUG("");
 		strob_strcpy(file_pathM, "");
 		strob_strcpy(file_linksource_pathM, "");
@@ -2674,6 +2657,10 @@ private: // ================================================================
 		o_path = path;
 		if (o_path) strob_strcpy(file_pathM, o_path);
 
+		if (verbose_levelM > SWC_VERBOSE_3) {
+			fprintf(stderr, "swpackage: processing %s\n", o_source);
+		}
+
 		//
 		// Source is req'd according to the synopsis on line 788.
 		// Do required Sanity checks.
@@ -2772,10 +2759,17 @@ private: // ================================================================
 		SWEXTDEF_DEBUG3("type path=[%s] source=[%s]", path, source);
 		SWEXTDEF_DEBUG3("type o_path=[%s] o_source=[%s]", o_path, o_source);
 		SWEXTDEF_DEBUG("");
-		if (fileArrayWasSetM[typeE] == 0) {	
+		if (fileArrayWasSetM[typeE] == 0) {
 			attach_filePaths(swdef, SW_A_source, source, path);
-			if (generate_ExplicitFileAttributes(source, source, st, 1)) return -1;
-		} else if (fileArrayM[typeE][0] == SW_ITYPE_d) {	
+			ret = generate_ExplicitFileAttributes(source, source, st, 1);
+			if (ret < 0) return -1;
+			if (ret > 0) {
+				// ignore file, such as a socket
+				delete swdef;
+				swdefFormedM = NULL;
+				return 1;
+			}
+		} else if (fileArrayM[typeE][0] == SW_ITYPE_d) {
 			//
 			// 5.2.13.3: Lines 831-836.
 			//
@@ -2787,7 +2781,14 @@ private: // ================================================================
 					attach_filePaths(swdef, SW_A_source, source, source);
 				}
 				SWEXTDEF_DEBUG("type d: case 1");
-				if (local_access(source, F_OK)) {
+				SWEXTDEF_DEBUG2("format type %d", swvarfs_get_format(swvarfsM));
+				if (
+					(
+					swvarfs_get_format(swvarfsM) != UINFILE_FILESYSTEM &&
+					(strcmp(source, ".") == 0 || strcmp(source, "./") == 0 || strcmp(source, "/") == 0)
+					) ||
+					local_access(source, F_OK)
+				) {
 					//
 					// not found or no access.
 					// Don't stat the source
@@ -2846,7 +2847,7 @@ private: // ================================================================
 				if (apply_default_values(fileArrayM[typeE][0]))
 					return -1;
 			}
-			if (generate_ExplicitFileAttributes(source, source, st, !(*do_no_stat))) return -1;
+			if (generate_ExplicitFileAttributes(source, source, st, !(*do_no_stat)) < 0) return -1;
 		} else if (fileArrayM[typeE][0] == SW_ITYPE_h) {	
 			//
 			// Hard Link: 5.2.13.3: Lines 837-840.
@@ -2913,7 +2914,7 @@ private: // ================================================================
 
 			attach_filePaths(swdef, "link_source", strob_str(file_linksource_pathM), (char*)(NULL));
 			attach_filePaths(swdef, SW_A_source, o_source, strob_str(file_pathM));
-			if (generate_ExplicitFileAttributes(source, (char *)NULL, st, !(*do_no_stat))) return -1;
+			if (generate_ExplicitFileAttributes(source, (char *)NULL, st, !(*do_no_stat)) < 0) return -1;
 
 		} else if (fileArrayM[typeE][0] == SW_ITYPE_s) {	
 			//
@@ -2940,7 +2941,7 @@ private: // ================================================================
 			if (apply_default_values(fileArrayM[typeE][0]))
 				return -1;
 			copy_file_cell(fileArrayM, fileArrayWasSetM, modeE, "0777");
-			if (generate_ExplicitFileAttributes(source, (char*)NULL, st, 0)) return -1;
+			if (generate_ExplicitFileAttributes(source, (char*)NULL, st, 0) < 0) return -1;
 			if (do_no_stat) *do_no_stat = 1;
 		} else {	
 			fprintf(stderr, "swpackage: bad file type in PSF file : '%s'.\n", fileArrayM[typeE]);
@@ -2963,7 +2964,7 @@ private: // ================================================================
 
 		SWEXTDEF_DEBUG("");
 		swdefFormedM = swdef;
-		return 0;	
+		return 0;
 	}
 };
 #endif
