@@ -2,7 +2,7 @@
  */
 
 /* 
-   Copyright (C) 2002  James H. Lowe, Jr.
+   Copyright (C) 2002,2008  James H. Lowe, Jr.
    All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
@@ -27,6 +27,18 @@
 
 #define swbis_free free
 
+static
+int
+f_strcmp(int sense, const void * vf1, const void * vf2)
+{
+        char * f1;
+        char * f2;
+        f1 = *((char**)(vf1));
+        f2 = *((char**)(vf2));
+        if (sense < 0 ) return -strcmp(f1, f2);
+        return strcmp(f1, f2);
+}
+
 STRAR *
 strar_copy_construct(STRAR * src)
 {
@@ -162,9 +174,11 @@ strar_qsort(STRAR * strar, int (*f_comp)(const void*, const void*))
 int
 strar_qsort_neg_strcmp(const void * vf1, const void * vf2)
 {
-        char * f1;
-        char * f2;
-        f1 = *((char**)(vf1));
-        f2 = *((char**)(vf2));
-        return -strcmp(f1, f2);
+	return f_strcmp(-1, vf1, vf2);
+}
+
+int
+strar_qsort_strcmp(const void * vf1, const void * vf2)
+{
+	return f_strcmp(1, vf1, vf2);
 }
