@@ -803,11 +803,7 @@ swgpg_get_package_signature_command(
 char *
 swgpg_get_package_signature(
 		SHCMD * sigcmd,
-		char * gpg_name,
-		char * gpg_path,
 		int * statusp,
-		int archive_format,
-		int header_flags,
 		char * wopt_passphrase_fd,
 		char * passfile,
 		int pkg_fd,
@@ -862,6 +858,7 @@ swgpg_get_package_signature(
 	if (pipe(passphrase)) exit(4);
 	pipe(filter);
 
+	E_DEBUG("");
 	does_use_agent = get_does_use_gpg_agent(cmd[0]);
 
 	/* shcmd_cmdvec_debug_show_to_file(cmd, stderr); 
@@ -899,6 +896,7 @@ swgpg_get_package_signature(
 	/*
 	 * Ask Passphrase
 	 */
+	E_DEBUG("");
 	pid[1] = swfork((sigset_t*)(NULL));
 	if (pid[1] < 0) return (char*)NULL;
 	if (pid[1] == 0) {
@@ -1017,59 +1015,42 @@ swgpg_get_package_signature(
 	/* 
 	 * Close the passphrase in the parent.
 	 */
+	E_DEBUG("");
 	close_passfd();
 	close(passphrase[1]);
 					/* filter[1] --> */
 	/*
 	 * Decode signing stream.
 	 */
+		/* FIXME this fork is not required */
 	pid[2] = swfork((sigset_t*)(NULL));
 	if (pid[2] < 0) return (char *)NULL;
 	if (pid[2] == 0) {
 		int flags = 0;
 		XFORMAT * xformat;
 		close_passfd();
-		xformat = xformat_open(
-				STDIN_FILENO, 
-				STDOUT_FILENO, 
-				archive_format);
-		flags = UINFILE_DETECT_FORCEUXFIOFD |
-			UINFILE_DETECT_NATIVE |
-			UINFILE_UXFIO_BUFTYPE_MEM;
-		xformat_set_tarheader_flags(xformat, header_flags);	
 		close(input[1]);
-	
-		if (xformat_open_archive_by_fd(xformat, 
-					input[0], 
-					flags, 0)) {
-			swlib_doif_writef(verboseG, SWC_VERBOSE_1, NULL, get_stderr_fd(),
-				"internal error opening archive in get_package_digests.\n");
-			_exit(4);
-		}
-
-		taruib_write_catalog_stream(
-					xformat,
-					filter[1],
-					1, /*version*/
-					0 /*verbose*/);
-	
+		swlib_pipe_pump(filter[1], input[0]);
 		close(input[0]);
 		close(filter[1]);
 		_exit(0);
 	}
 	close(input[0]);
 	close(filter[1]);
+	E_DEBUG("");
 					/* <<-- input[0] */
 					/* input[1] --> */
 	/*
-	 * Write whole package.
+	 * Write signed stream
 	 */
+	E_DEBUG("");
 	pid[3] = swfork((sigset_t*)(NULL));
 	if (pid[3] < 0) return (char *)NULL;
 	if (pid[3] == 0) {
+		int ofd1;
 		close_passfd();
 		close(input[0]);
-		int ofd1;
+
 		ofd1 =  uxfio_opendup(input[1], UXFIO_BUFTYPE_NOBUF);
 		uxfio_fcntl(ofd1, 
 				UXFIO_F_SET_OUTPUT_BLOCK_SIZE, 
@@ -1077,23 +1058,13 @@ swgpg_get_package_signature(
 
 		swlib_pipe_pump(ofd1, pkg_fd);
 
-		/*
-		write_serial_package(g_psf, swexdist, 
-			ofd1, 
-			0,
-			0,
-			g_wopt_dir_numeric_owner,
-			g_wopt_catalog_numeric_owner,
-			g_wopt_numeric_owner, no_front_dir,
-			0
-			);
-		*/
 		uxfio_close(ofd1);
 		close(output[0]);
 		_exit(0);
 	}
 	close(input[1]);
 
+	E_DEBUG("");
 	sigfd = swlib_open_memfd();
 	
 	ret = swlib_pump_amount(sigfd, output[0], 1024);
@@ -1101,6 +1072,7 @@ swgpg_get_package_signature(
 	if (ret < 0 || ret > 1000) return (char*)NULL;
 	uxfio_write(sigfd, (void*)nullbyte, 1);	
 
+	E_DEBUG("");
 	if (swlib_wait_on_all_pids(pid, 4, status, 
 				WNOHANG, verboseG - 2) < 0) {
 		return NULL;
@@ -1108,6 +1080,7 @@ swgpg_get_package_signature(
 		/* swexdist->setErrorCode(18004, NULL); */
 	}
 
+	E_DEBUG("");
 	if (pid[0] < 0) {
 		ret =  WEXITSTATUS(status[0]);
 	} else {
@@ -1116,10 +1089,12 @@ swgpg_get_package_signature(
 	*statusp = ret;
 	if (ret) return (char*)NULL;
 
+	E_DEBUG("");
 	uxfio_get_dynamic_buffer(sigfd, &fdmem, (int*)NULL, (int*)NULL);
 	sig = strdup(fdmem);
 	uxfio_close(sigfd);	
 	swlib_doif_writef(verboseG, SWPACKAGE_VERBOSE_V1, NULL, get_stderr_fd(),
 		"Generating package signature .... Done.\n");
+	E_DEBUG("");
 	return sig;
 }
