@@ -21,6 +21,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
+#define FILENEEDDEBUG 1
+#undef FILENEEDDEBUG
 
 #include "swuser_config.h"
 #include <stdio.h>
@@ -891,10 +893,12 @@ swheader_fileobject2filehdr(SWHEADER * fileheader, struct new_cpio_header * file
 		} else if (strcmp(keyword, "path") == 0) {
 			ahsStaticSetTarFilename(file_hdr, value);
 		} else if (strcmp(keyword, "uid") == 0) {
+			E_DEBUG("uid");
 			file_hdr->c_uid = (unsigned long)atol(value);		
 			(file_hdr->usage_maskM) |= TARU_UM_UID;
 			did_uid = 1;
 		} else if (strcmp(keyword, "gid") == 0) {
+			E_DEBUG("gid");
 			file_hdr->c_gid = (unsigned long)atol(value);		
 			(file_hdr->usage_maskM) |= TARU_UM_GID;
 			did_gid = 1;
@@ -902,10 +906,12 @@ swheader_fileobject2filehdr(SWHEADER * fileheader, struct new_cpio_header * file
 			ahsStaticSetTarLinkname(file_hdr, value);
 			did_link_source = 1;
 		} else if (strcmp(keyword, "owner") == 0) {
+			E_DEBUG("owner");
 			ahsStaticSetTarUsername(file_hdr, value);
 			did_uname = 1;
 			(file_hdr->usage_maskM) |= TARU_UM_OWNER;
 		} else if (strcmp(keyword, "group") == 0) {
+			E_DEBUG("group");
 			ahsStaticSetTarGroupname(file_hdr, value);
 			did_gname = 1;
 			(file_hdr->usage_maskM) |= TARU_UM_GROUP;
