@@ -159,7 +159,7 @@ init_magic_header(SWICOL * swicol)
 	/* This makes a string 513 bytes long */
 	tmp = strob_open(10);
 	for (i=0; i<9; i++)
-		strob_sprintf(tmp, 1, "%s\n", "#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_#_##");
+		strob_sprintf(tmp, 1, "%s\n", "# # # # # # # # # # # # # # # # # # # # # # # # # # # ##");
 	swicol->magic_headerM = strob_release(tmp);
 	if (strlen(swicol->magic_headerM) < 513) exit(42);
 }
@@ -416,7 +416,12 @@ print_task_script(SWICOL * swicol, STROB * command, uintmax_t size, char * dir,
 		"			;;\n"
 		"	esac\n"
 		"%s"		/* Task main script */
-		"	exit \"$sw_retval\"\n"
+		"case \"$sw_retval\" in\n"
+		"	\"\") echo \"%s:\" Warning: sw_retval is not set for task script: %s 1>&2\n"
+		"		sw_retval=0\n"
+		"		;;\n"
+		"esac\n"
+		"exit \"$sw_retval\"\n"
 		"); sw_retval=$?\n"
 		"%s\n"			/* SWI_TASK_ENDS */
 		"exit $sw_retval\n"	/* exit the task shell */
@@ -430,6 +435,8 @@ print_task_script(SWICOL * swicol, STROB * command, uintmax_t size, char * dir,
 		ablocks,
 		TEVENT(2, -1, SW_INTERNAL_ERROR, "No such directory or no access: $swxdir"),
 		task_script,
+		swlib_utilname_get(),
+		task_desc,
 		TEVENT(2, -1, SWI_TASK_ENDS, strob_str(enddesc))
 		);
 
