@@ -2161,7 +2161,6 @@ swlib_wait_on_all_pids(pid_t * pid, int num, int * status,
 						strerror(errno));
 					status[i] = 0;
 					pid[i] = -pid[i];
-					/* return -1; */
 				} else if (wret == 0) {
 					swlib_doif_writef(verbose_level,
 						SWC_VERBOSE_SWIDB,
@@ -2308,10 +2307,6 @@ swlib_shcmd_output_strob(STROB * output, SHCMD ** cmdvec)
 	strob_strcpy(output, "");
 	fd = swlib_shcmd_output_fd(cmdvec);
 	if (fd < 0) return -1;
-	/*
-	uxfio_lseek(fd, 0, SEEK_SET);
-	uxfio_write(fd, "", 1);
-	*/
 	if (uxfio_get_dynamic_buffer(fd, &base, &buffer_len, &data_len) < 0)
 		return -1;
 	strob_strncat(output, base, data_len);
@@ -2355,7 +2350,6 @@ swlib_open_memfd(void)
 {
 	int fd = uxfio_open("/dev/null", O_RDONLY, 0);
 	if (fd < 0) return fd;
-	/* swi_assert_value(fd >= 0, __FILE__, __LINE__); */
 	uxfio_fcntl(fd, UXFIO_F_SET_BUFACTIVE, UXFIO_ON);
 	uxfio_fcntl(fd, UXFIO_F_SET_BUFTYPE, UXFIO_BUFTYPE_DYNAMIC_MEM);
 	return fd;
