@@ -804,7 +804,6 @@ swi_decode_catalog(SWI * swi)
 	int retval;
 	int is_catalog;
 	char * name;
-	int header_bytes;
 	int swdef_ret = 0;
 	int swdeffile_type;
 	int ifd;
@@ -825,7 +824,6 @@ swi_decode_catalog(SWI * swi)
 	}
 
 	while(name) {
-		header_bytes = ret;
 		E_DEBUG("");
 		if (xformat_is_end_of_archive(xformat)){
 			/* 
