@@ -6,7 +6,7 @@
    COPYING TERMS AND CONDITIONS:
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,7 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include "swuser_config.h"
 #include <stdio.h>
@@ -53,6 +53,7 @@ swi_vbase_init(void * derived, int type, SWHEADER * index, SWPATH_EX * current)
 	base->verboseM = 1;
  	base->swveridM = swverid_open(NULL, NULL); /* Fixme */
 	base->id_endM = SWI_BASE_ID_END;
+	
 	if (index && current) {
 		/*
 		 * This block of code is executed for filesets and products
@@ -77,6 +78,10 @@ swi_vbase_init(void * derived, int type, SWHEADER * index, SWPATH_EX * current)
 		}
 		SWLIB_ASSERT(tag != NULL);
 		base->tagM = strdup(tag);
+	} else {
+		/* initialize */
+		base->header_indexM = 0; /* 0 is the unset value */
+		base->tagM = strdup("");
 	}
 }
 
