@@ -661,12 +661,10 @@ taru_write_archive_member_header(TARU * taru,
 	} else if (archive_format_out == arf_ustar ||
 		archive_format_out == arf_tar) {
 		if (linkrecord_active) {
-			E_DEBUG("");
-			nfound=0;	
-			if (file_hdr->c_nlink > 1)
-				E_DEBUG("");
-				link_record_buf = 
-				hllist_find_file_entry(linkrecord, 
+			   E_DEBUG("");
+			   nfound=0;	
+			   if (file_hdr->c_nlink > 1) {
+				link_record_buf = hllist_find_file_entry(linkrecord, 
 					(dev_t) makedev(file_hdr->c_dev_maj,\
 				   	    file_hdr->c_dev_min),
 	 				file_hdr->c_ino,
@@ -695,6 +693,7 @@ taru_write_archive_member_header(TARU * taru,
 				} else {
 					E_DEBUG("not found");
 				}
+			    }
 			}
 			else {
 				E_DEBUG("");
