@@ -913,7 +913,7 @@ report_dig_error(STROB * buf, int msgfd, int efd, char * digname, int dlen, int
 		swlib_writef(efd, buf, "%s: package integrity error: archive %s\n", swlib_utilname_get(), digname);
 	}
 	if (dlen == 0) {
-		fprintf(stderr, "%s: Package does not contain an %s digest.\n", swlib_utilname_get(), digname);
+		fprintf(stderr, "%s: Warning: %s digest not present\n", swlib_utilname_get(), digname);
 	}
 
 }
