@@ -2,12 +2,12 @@
  */
 
 /*
- * Copyright (C) 2003 James H. Lowe, Jr.  <jhlowe@acm.org>
+ * Copyright (C) 2003,2007 James H. Lowe, Jr.  <jhlowe@acm.org>
  *
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 #include "swuser_config.h"
@@ -33,13 +33,18 @@
 #include <errno.h>
 #include "ahs.h"
 #include "uinfile.h"
+#include "fnmatch_u.h"
 #include "swlib.h"
 #include "swutilname.h"
 #include "swgp.h"
 #include "swfork.h"
-
+#include "u_ar.h"
+#include "ls_list.h"
 
 #include "debug_config.h"
+#define UINFILENEEDDEBUG 1
+#undef UINFILENEEDDEBUG 
+
 #ifdef UINFILENEEDDEBUG
 #define UINFILE_E_DEBUG(format) SWBISERROR("UINFILE DEBUG: ", format)
 #define UINFILE_E_DEBUG2(format, arg) \
@@ -55,6 +60,44 @@
 #define UINFILE_I_INIT_READ_SIZE 124
 #define UINFILE_I_TAR_BLOCK_SIZE 512
 
+static
+int
+does_have_name_version(char * buf)
+{
+	char * s;
+	UINFILE_E_DEBUG2("buf=[%s]", buf);
+	s = strchr(buf, '-');
+	if (s == NULL) return 0;
+	s++;
+	if (
+		isdigit((int)(*s)) &&
+		!isdigit((int)(*buf))
+	) {
+		UINFILE_E_DEBUG("return 1");
+		return 1;
+	} else {
+		UINFILE_E_DEBUG("return 0");
+		return 0;
+	}
+}
+
+static
+int
+ar_get_size(unsigned char * buf, int * value)
+{
+	int result;
+	struct ar_hdr a;
+	/* buf points to a GNU ar header */
+
+	memcpy(&a, buf, sizeof(a));	
+	if (memcmp(a.ar_fmag, ARFMAG, 2) != 0)
+		return -1;
+	a.ar_fmag[0] = '\0';
+	*value = swlib_atoi((char*)(a.ar_size), &result);
+	if (result != 0)
+		return -1;
+	return 0;
+}
 
 static 
 int
@@ -66,7 +109,7 @@ determine_if_has_leading_slash(UINFORMAT * uinformat, char * buffer)
 		int eoa;
 		struct new_cpio_header * file_hdr = taru_make_header();
 		taru_read_in_tar_header2(uinformat->taruM, file_hdr,
-							-1, buffer, &eoa, 0);
+					-1, buffer, &eoa, 0, TARRECORDSIZE);
 		if (*ahsStaticGetTarFilename(file_hdr) == '/') ret = 1;
 		taru_free_header(file_hdr);
 	} else if (uinformat->typeM == CPIO_POSIX_FILEFORMAT) {
@@ -303,7 +346,9 @@ uinfile_handle_return(UINFORMAT * uinformat, int oflags, int uxfio_buffer_type)
 		uinformat->fdM = uxfio_opendup(uinformat->fdM,
 							uxfio_buffer_type);
 		uxfio_fcntl(uinformat->fdM, UXFIO_F_SET_CANSEEK, 0);
+		/*
 		UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(uinformat->fdM));
+		*/
 	} else if ( 
 		(oflags & UINFILE_DETECT_FORCE_SEEK &&
 					uxfio_espipe(uinformat->fdM) && 
@@ -313,7 +358,9 @@ uinfile_handle_return(UINFORMAT * uinformat, int oflags, int uxfio_buffer_type)
 		UINFILE_E_DEBUG("HERE");
 		uinformat->fdM = uxfio_opendup(uinformat->fdM,
 							uxfio_buffer_type);
+		/*
 		UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(uinformat->fdM));
+		*/
 	} else if ( 
 		(oflags & UINFILE_DETECT_FORCE_SEEK &&
 					uxfio_espipe(uinformat->fdM) && 
@@ -323,7 +370,9 @@ uinfile_handle_return(UINFORMAT * uinformat, int oflags, int uxfio_buffer_type)
 		UINFILE_E_DEBUG("HERE");
 		uxfio_fcntl(uinformat->fdM, UXFIO_F_SET_BUFTYPE,
 							UXFIO_BUFTYPE_MEM);
+		/*
 		UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(uinformat->fdM));
+		*/
 	} else {
 		UINFILE_E_DEBUG("HERE");
 		;
@@ -346,59 +395,87 @@ uinfile_i_open(char *filename, int oflag, mode_t mode, UINFORMAT ** uinformat,
 	int fd, pipe_fd[2], dupfd;
 	int refd;
 	int zpipe[2];
+	unsigned long tar_hdr_sum;
 	pid_t pid;
-	char buffer[512];
+	char xbuffer[1025];
 	char gzmagic[] = UINFILE_MAGIC_gz;
 	char Zmagic[] = UINFILE_MAGIC_Z;
 	char rpmmagic[] = UINFILE_MAGIC_rpm;
 	char bz2magic[] = UINFILE_MAGIC_bz2;
 	char debmagic[] = UINFILE_MAGIC_deb;
+	char slackmagic[] = UINFILE_MAGIC_slack;
 	char *zcat_command = (char *) (NULL);
 	pid_t a_pid;
-	int pump_amount = -1;
+	intmax_t pump_amount = -1;
 	int zret = 0;	
 	int forcetar;
 	int doieee;
+	int dodebcontrol;
+	int dodebcontext;
+	int dodebdata;
 	int doarb;
 	int dounrpminstall;
 	int douncpio;
+	int donative;
 	int peeklen;
-	int control_gz_size = 0;
+	int deb_gz_size = 0;
+	int deb_gz_offset = 0;
 	int val1;
 	int val2;
-	char * s_size;
-
-	UINFILE_E_DEBUG("");
-	*uinformat=(UINFORMAT *)malloc(sizeof(UINFORMAT));
-	if (!(*uinformat)) {
-		return -1;
-	}
-	(*uinformat)->underlying_fdM = -1;
-	(*uinformat)->current_pos_ = 0;
-	(*uinformat)->zcat_cmdM = zcat_command;
-	(*uinformat)->file_hdrM=ahsStaticCreateFilehdr();
-	(*uinformat)->verboseM = 0;
-	(*uinformat)->has_leading_slashM = 0;
-	(*uinformat)->ztypeM = UINFILE_COMPRESSED_NA;
-	(*uinformat)->typeM = UNKNOWN_FILEFORMAT;
-	memset((*uinformat)->type_revisionM, '\0', sizeof((*uinformat)->type_revisionM));
-	(*uinformat)->swpathM = (SWPATH*)(NULL);
-	(*uinformat)->taruM = taru_create();
-	sigemptyset(&((*uinformat)->blockmask_));
-	sigemptyset(&((*uinformat)->defaultmask_));
-	sigaddset(&((*uinformat)->blockmask_), SIGALRM);
-	sigaddset(&((*uinformat)->defaultmask_), SIGTERM);
-	sigaddset(&((*uinformat)->defaultmask_), SIGINT);
-	sigaddset(&((*uinformat)->defaultmask_), SIGPIPE);
 
+	UINFILE_E_DEBUG("BEGIN ****************************** ");
+	UINFILE_E_DEBUG2("xdupfd = %d", xdupfd);
+	dodebcontrol = oflags & UINFILE_DETECT_DEB_CONTROL;
+	dodebcontext = oflags & UINFILE_DETECT_DEB_CONTEXT;
+	dodebdata = oflags & UINFILE_DETECT_DEB_DATA;
+	UINFILE_E_DEBUG2("dodebdata=%d", dodebdata);
+	UINFILE_E_DEBUG2("dodebcontext=%d", dodebcontext);
+	UINFILE_E_DEBUG2("dodebcontrol=%d", dodebcontrol);
 	forcetar = oflags & UINFILE_DETECT_OTARFORCE;
 	doieee = oflags & UINFILE_DETECT_IEEE;
 	doarb = oflags & UINFILE_DETECT_ARBITRARY_DATA;
 	dounrpminstall = oflags & UINFILE_DETECT_UNRPM;
+/* JL test dounrpminstall = 0;  */
 	douncpio = oflags & UINFILE_DETECT_UNCPIO;
+	donative = oflags & UINFILE_DETECT_NATIVE;
+	
+	UINFILE_E_DEBUG2("forcetar=%d", forcetar);
+	UINFILE_E_DEBUG2("donative=%d", donative);
+
+	if (dodebdata) {
+		;
+		/* Special case, the uinfile struct is already created */
+	} else {
+		*uinformat=(UINFORMAT *)malloc(sizeof(UINFORMAT));
+		if (!(*uinformat)) {
+			return -1;
+		}
+		(*uinformat)->underlying_fdM = -1;
+		(*uinformat)->current_pos_ = 0;
+		(*uinformat)->zcat_cmdM = zcat_command;
+		(*uinformat)->file_hdrM=ahsStaticCreateFilehdr();
+		(*uinformat)->verboseM = 0;
+		(*uinformat)->has_leading_slashM = 0;
+		(*uinformat)->ztypeM = UINFILE_COMPRESSED_NA;
+		(*uinformat)->typeM = UNKNOWN_FILEFORMAT;
+		memset((*uinformat)->type_revisionM, '\0', sizeof((*uinformat)->type_revisionM));
+		(*uinformat)->swpathM = (SWPATH*)(NULL);
+		(*uinformat)->taruM = taru_create();
+		(*uinformat)->slackheaderM = NULL;
+		UINFILE_E_DEBUG("seting deb_file_fd = -1");
+		(*uinformat)->deb_file_fd_ = -1;
+		(*uinformat)->n_deb_peeked_bytesM = 0;
+		(*uinformat)->pathname_prefixM = NULL;
+		sigemptyset(&((*uinformat)->blockmask_));
+		sigemptyset(&((*uinformat)->defaultmask_));
+		sigaddset(&((*uinformat)->blockmask_), SIGALRM);
+		sigaddset(&((*uinformat)->defaultmask_), SIGTERM);
+		sigaddset(&((*uinformat)->defaultmask_), SIGINT);
+		sigaddset(&((*uinformat)->defaultmask_), SIGPIPE);
+		for (i=0; i < (int)(sizeof((*uinformat)->pidlistM)/sizeof(int)); i++)
+				(*uinformat)->pidlistM[i] = 0;
+	}
 
-	for (i=0; i < (int)(sizeof((*uinformat)->pidlistM)/sizeof(int)); i++)
-					(*uinformat)->pidlistM[i] = 0;
 
 	/* ------------------------------------------------------------- */
 	/*            Open file or pipe                                  */
@@ -425,9 +502,13 @@ uinfile_i_open(char *filename, int oflag, mode_t mode, UINFORMAT ** uinformat,
 				/*
 				 * unseekable pipe
 				 */
+				/*
 				UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(fd));
+				*/
 				fd = uxfio_opendup(dupfd, uxfio_buffer_type);
+				/*
 				UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(fd));
+				*/
 				if (fd < 0) {
 					fprintf(stderr,"uinfile.c: uxfio_opendup failed\n");
 					return -1;
@@ -442,9 +523,11 @@ uinfile_i_open(char *filename, int oflag, mode_t mode, UINFORMAT ** uinformat,
 			if (ret >= 0) {
 				UINFILE_E_DEBUG2("HERE current_pos_=%d", (int)((*uinformat)->current_pos_));
 				(*uinformat)->current_pos_ = ret;
+				/*
 				if ((*uinformat)->current_pos_) {
 					fprintf(stderr,"uinfile.c: warning: current_pos_ not equal to zero\n");
 				}
+				*/
 			} else {
 				/*
 				fprintf(stderr,"uinfile.c: warning: lseek failed\n");
@@ -460,18 +543,28 @@ uinfile_i_open(char *filename, int oflag, mode_t mode, UINFORMAT ** uinformat,
 			UINFILE_E_DEBUG("HERE error.\n");
 			return fd;
 		}
+		dupfd = fd;
 		(*uinformat)->did_dupeM=0;
 	}
-	refd=fd;
+	refd = fd;
 	(*uinformat)->underlying_fdM = refd;
 LABEL_REREAD:
 	(*uinformat)->fdM = refd;
+	if ((*uinformat)->typeM == DEB_FILEFORMAT && dodebdata == 0) {
+		/* FIXME, should go through handle_return() */
+		return refd;
+	}
 
 	/*
 	 * Initial read on the serial access archive.
 	 * Fixme:
 	 */
-	if (doarb == 0) {
+	if (dodebdata) {
+		UINFILE_E_DEBUG("HERE");
+		peeklen = SARHDR +
+				 1 + /* 1 for possible padding */
+				 strlen(UINFILE_MAGIC_gz);  /* for gz magic */
+	} else if (doarb == 0) {
 		UINFILE_E_DEBUG("HERE");
 		peeklen = UINFILE_I_INIT_READ_SIZE;
 	} else {
@@ -482,10 +575,12 @@ LABEL_REREAD:
 		peeklen = 3;
 	}	
 
-	memset(buffer, '\0', sizeof(buffer));
+	memset(xbuffer, '\0', sizeof(xbuffer));
 
-	if ((ret=taru_tape_buffered_read(refd, buffer, peeklen)) != peeklen) {
-		if (ret) fprintf(stderr,
+	UINFILE_E_DEBUG3("HERE refd=%d  peeklen=%d", refd, peeklen);
+	if ((ret=taru_tape_buffered_read(refd, xbuffer, peeklen)) != peeklen) {
+		UINFILE_E_DEBUG("ERROR HERE");
+		if (ret <= 0) fprintf(stderr,
 		"%s: error in initial read. request=%d return=%d\n",
 			swlib_utilname_get(), peeklen,  ret);
 		return -1;
@@ -498,7 +593,50 @@ LABEL_REREAD:
 	if (doarb == 0) {
 		if (0) {
 			;
-		} else if (!memcmp(buffer, rpmmagic, 4) && !forcetar) {
+		} else if (dodebdata) {
+			UINFILE_E_DEBUG("in dodebdata");
+			/* the ar header of data.tar.gz should be in xbuffer */
+			UINFILE_E_DEBUG("type: DEB (data)");
+			ret = ar_get_size((unsigned char*)xbuffer, &val1);
+			if ( ret != 0 ) {
+				/* this may be caused by alignment byte padding in the ar format,
+				   so try it again, if it fails again only then your screwed. */
+				ret = ar_get_size((unsigned char*)xbuffer+1, &val1);
+				if ( ret != 0 ) {
+					fprintf(stderr,
+						"%s: ar archive error %s: at line %d\n", swlib_utilname_get(), __FILE__, __LINE__);
+					return -1;
+				}
+				UINFILE_E_DEBUG("in dodebdata with 1 byte of padding");
+				deb_gz_offset = SARHDR + 1;
+				/* for example "\n\037\213\b"
+				use offset of deb_gz_offset - SARHDR with write size of 3 */
+
+				(*uinformat)->n_deb_peeked_bytesM = strlen(UINFILE_MAGIC_gz);
+				
+			} else {
+				UINFILE_E_DEBUG("in dodebdata with no padding");
+				deb_gz_offset = SARHDR;
+				/* for example "\037\213\b\xNN"
+				use offset of deb_gz_offset - SARHDR with write size of 4 */
+
+				(*uinformat)->n_deb_peeked_bytesM = strlen(UINFILE_MAGIC_gz) + 1;
+				;
+			}
+			memcpy((void*)((*uinformat)->deb_peeked_bytesM), xbuffer+SARHDR, peeklen - SARHDR);
+	
+			deb_gz_size = val1;
+			UINFILE_E_DEBUG2("deb_gz_size is %d", val1);
+			ret = uxfio_lseek(refd, (off_t)(0), SEEK_CUR);
+			if (ret < 0) {
+				fprintf(stderr, "%s: ar archive error %s: at line %d\n", swlib_utilname_get(), __FILE__, __LINE__);
+				return -1;
+			}
+			(*uinformat)->current_pos_ = ret;
+			UINFILE_E_DEBUG2("current_pos_ is %d", ret);
+
+		} else if (!memcmp(xbuffer, rpmmagic, 4) && !forcetar) {
+			UINFILE_E_DEBUG("type: RPM");
 			if (dounrpminstall == 0 && doieee == 0) {
 				/*
 				 * This is the old historic response to
@@ -507,7 +645,9 @@ LABEL_REREAD:
 				(*uinformat)->typeM=RPMRHS_FILEFORMAT;
 				ret = uinfile_handle_return(*uinformat, oflags,
 							uxfio_buffer_type);
+				/*
 				UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(ret));
+				*/
 				return ret;
 			} else if (dounrpminstall) {
 				/*
@@ -525,23 +665,26 @@ LABEL_REREAD:
 				SWLIB_EXCEPTION("");
 				return -1;
 			}
-		} else if (douncpio && ((strncmp(buffer, "0707", 4) == 0) &&
-				(1/*strncmp(buffer + 257, TMAGIC, 5)*/))) {
+		} else if (douncpio && ((strncmp(xbuffer, "0707", 4) == 0) &&
+				(1/*strncmp(xbuffer + 257, TMAGIC, 5)*/))) {
 			/*
 			 * Model the cpio format as a compression type since this
 			 * fits with the code below, e.g. instead of calling
 			 * ''gzip -d'', call ''arf2arf -H ustar''
 			 */
+			UINFILE_E_DEBUG("type: 0707");
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_CPIO;
-		} else if (!strncmp(buffer, "0707", 4) &&
-				(1/*strncmp(buffer + 257, TMAGIC, 5)*/) &&
+		} else if (!strncmp(xbuffer, "0707", 4) &&
+				(1/*strncmp(xbuffer + 257, TMAGIC, 5)*/) &&
 								!forcetar) {
-			UINFILE_E_DEBUG("HERE its 0707");
-			if (!strncmp(buffer, "070702", 6)) { /* new ascii crc */
+			if (!strncmp(xbuffer, "070702", 6)) { /* new ascii crc */
+				UINFILE_E_DEBUG("type: 070702");
 				(*uinformat)->typeM = CPIO_CRC_FILEFORMAT;
-			} else if (!strncmp(buffer, "070701", 6)) {/*new ascii*/
+			} else if (!strncmp(xbuffer, "070701", 6)) {/*new ascii*/
+				UINFILE_E_DEBUG("type: 070701");
 				(*uinformat)->typeM = CPIO_NEWC_FILEFORMAT;
-			} else if (!strncmp(buffer, "070707", 6)) {/* POSIX.1 */
+			} else if (!strncmp(xbuffer, "070707", 6)) {/* POSIX.1 */
+				UINFILE_E_DEBUG("type: 070707");
 				(*uinformat)->typeM = CPIO_POSIX_FILEFORMAT;
 			} else {
 				SWLIB_EXCEPTION("");
@@ -549,50 +692,356 @@ LABEL_REREAD:
 			}
 			(*uinformat)->has_leading_slashM =
 				determine_if_has_leading_slash(*uinformat,
-									buffer);
+									xbuffer);
 			ret =  uinfile_handle_return(*uinformat, oflags,
 							uxfio_buffer_type);
 			if (ret > 0 && doieee) {
 				ret = uinfile_detect_ieee(*uinformat, oflags);
 			}
-			UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(ret));
 			return ret;
-		} else if (!memcmp(buffer, (void *) gzmagic, 2)) {
+		} else if (!memcmp(xbuffer, (void *) gzmagic, 2)) {
 						/* gzipped compression */
+			UINFILE_E_DEBUG("type: COMPRESSED_GZ");
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_GZ;
-		} else if (!memcmp(buffer, (void *) bz2magic, 3)) {
+		} else if (!memcmp(xbuffer, (void *) bz2magic, 3)) {
 							/* bz2 compression */
+			UINFILE_E_DEBUG("type: COMPRESSED_BZ2");
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_BZ2;
-		} else if (!memcmp(buffer, (void *) Zmagic, 2)) {
+		} else if (!memcmp(xbuffer, (void *) Zmagic, 2)) {
 							/* Unix compress */
+			UINFILE_E_DEBUG("type: COMPRESSED_Z");
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_Z;
-		} else if (!memcmp(buffer, (void *)debmagic, strlen(UINFILE_MAGIC_deb))) {
-							/* deb package */
+		} else if ( donative == 0 &&
+			dounrpminstall == 0 &&
+			memcmp(xbuffer, (void *)debmagic, strlen(UINFILE_MAGIC_deb)) == 0
+		) {
+			UINFILE_E_DEBUG("type: DEB");
+			/* Great, its a deb package */
+
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_DEB;
+			(*uinformat)->typeM = DEB_FILEFORMAT;
+			UINFILE_E_DEBUG2("seting deb_file_fd = %d", dupfd);
+			(*uinformat)->deb_file_fd_ = dupfd;
+
+			/* Store the 2.0\n which is the data of the first
+			   archive member */
+	
 			strncpy((*uinformat)->type_revisionM,
-				((char*)buffer)+68, 4);
-			/*
-			 * Now read some more bytes to include the gzip magic of
-			 * the control.tar.gz file
-			 */
-			if ((ret=taru_tape_buffered_read(refd, buffer+peeklen,
-					UINFILE_DEB_CONTROL_OFFSET - peeklen + 4)) != 
-				UINFILE_DEB_CONTROL_OFFSET - peeklen + 4) {
-				fprintf(stderr,
-				"%s: error in second initial read. request=%d return=%d\n",
-					swlib_utilname_get(), peeklen,  ret);
+				((char*)xbuffer)+68, 4);
+			(*uinformat)->type_revisionM[4] = '\0';
+
+			/* Now read some more bytes to include the gzip magic of
+			   the control.tar.gz file */
+
+			ret = ar_get_size((unsigned char*)xbuffer+8, &val1);
+			if ( ret != 0 || val1 != 4 ) {
+				fprintf(stderr, "%s: deb format bad size value\n", swlib_utilname_get());
 				return -1;
 			}
+
+			/* Now we know for certain where the control.tar.gz file begins,
+			   make sure we have peek'ed that far into the file and confirm
+			   the gz magic is there */
+
+			deb_gz_offset = SARMAG + SARHDR + val1 + SARHDR;  /* val1 must be 4 */
+			if (peeklen < deb_gz_offset + 3) {
+				/* read further into the file */
+
+				ret = taru_tape_buffered_read(refd, xbuffer+peeklen, deb_gz_offset+3 - peeklen);
+				if (ret != deb_gz_offset+3 - peeklen) {
+					fprintf(stderr,
+					"%s: error in second initial read. request=%d return=%d\n",
+						swlib_utilname_get(), peeklen,  ret);
+					return -1;
+				}
+			}
+			
+			/* look for gzmagic */
+
+			if (memcmp(((unsigned char*)xbuffer)+deb_gz_offset, (void *)gzmagic, 3) != 0) {
+				fprintf(stderr, "%s: gz magic not found for control.tar.gz\n", swlib_utilname_get());
+				return -1;
+			}
+		
+			/* Now get the size of the compressed control.tar.gz file */
+	
+			ret = ar_get_size(
+					(unsigned char*)xbuffer + SARMAG + SARHDR + val1,  /* point to header beginning */
+					&val2);
+			if (ret != 0) {
+				fprintf(stderr, "%s: bad size conversion\n", swlib_utilname_get());
+				return -1;
+			}
+
+			if (uxfio_lseek(refd, (off_t)deb_gz_offset, SEEK_SET) != deb_gz_offset) {
+				fprintf(stderr, "%s: %s: uxfio_lseek error. 001.3 pos=%d\n",
+					swlib_utilname_get(), __FILE__, ((*uinformat)->current_pos_)); 
+				return -1;
+			}
+			/* this is important */
+			deb_gz_size = val2;
+
+			(*uinformat)->current_pos_ = deb_gz_offset;
+
+			/* Now continue as a compressed file */
+			;
+		} else if (doieee == 0 &&
+			donative == 0 &&
+			dodebcontext == 0 &&
+			dodebcontrol == 0 &&
+			dounrpminstall == 0 &&
+			dodebdata == 0 &&
+			does_have_name_version(xbuffer)
+		) {
+			STROB * tmp;
+			STROB * path_prefix;
+			SWVARFS * swvarfs;
+			struct stat st;
+			char * name;
+			int found;
+			int xeoa;
+
+			UINFILE_E_DEBUG("type: plain source tarball");
+			UINFILE_E_DEBUG("");
+			tmp = strob_open(32);
+
+			ret = taru_tape_buffered_read(refd, xbuffer+peeklen, sizeof(xbuffer) - peeklen);
+			if (ret != (int)sizeof(xbuffer) - (int)peeklen) {
+				return -1;
+			}
+			UINFILE_E_DEBUG2("ret=%d", ret);
+		
+			ret = taru_read_in_tar_header2((*uinformat)->taruM,
+							(*uinformat)->file_hdrM,
+							-1 /* fd */,
+							xbuffer,
+							&xeoa, 0, TARRECORDSIZE);
+			if (ret != TARRECORDSIZE) {
+				return -1;
+			}
+
+			/* xbuffer now contains what should be the leading path prefix of the
+			   plain tarball */
+
+			taru_print_tar_ls_list(tmp, (*uinformat)->file_hdrM, LS_LIST_VERBOSE_L2);
+			/* fprintf(stderr, "JL [%s]\n", strob_str(tmp)); */
+
+			(*uinformat)->slackheaderM = malloc(TARRECORDSIZE);
+			if ((*uinformat)->slackheaderM == NULL) return -1;
+			memcpy((*uinformat)->slackheaderM, xbuffer, TARRECORDSIZE);
+
+			path_prefix = strob_open(32);
+			UINFILE_E_DEBUG("");
+
+			/* read the entire package into the buffered file descriptor */
+			while ((ret=uxfio_read(refd, xbuffer, TARRECORDSIZE)) > 0); 
+			if (ret < 0) {
+				SWBIS_E_FAIL("read error");
+				return -1;
+			}
+
+			/* now seek back to beginning  */
+			UINFILE_E_DEBUG("");
+
+			ret = uxfio_lseek(refd, 0, SEEK_SET);
+			if (ret != 0) {
+				SWBIS_E_FAIL("");
+				return -1;
+			}
+
+			UINFILE_E_DEBUG("");
+			/* Now read the package to determine the path prefix */
+			swvarfs = swvarfs_opendup(refd, UINFILE_DETECT_NATIVE, (mode_t)0);
+			UINFILE_E_DEBUG("");
+			if (swvarfs == NULL) {
+				SWBIS_E_FAIL("");
+				UINFILE_E_DEBUG("");
+				return -1;
+			}
+			UINFILE_E_DEBUG("");
+			found = 0;
+			while ((name=swvarfs_get_next_dirent(swvarfs, &st)) != NULL && strlen(name)) {
+				/* While we are reading the files, store the leading path prefix
+				   and determine if it is the same for every file. */
+				if (path_prefix &&
+				    strob_strlen(path_prefix) < 4
+				) {
+					strob_strcpy(path_prefix, name);	
+					swlib_unix_dirtrunc(path_prefix);
+				} if (path_prefix &&
+					strob_strlen(path_prefix) >= 4 &&
+					strstr(name, strob_str(path_prefix)) != name
+				) {
+					strob_close(path_prefix);
+					path_prefix = NULL;
+				}
+				UINFILE_E_DEBUG2("||||||| [%s]", name);
+			}
+
+			UINFILE_E_DEBUG("");
+			swvarfs_close(swvarfs);
+			ret = uxfio_lseek(refd, 0, SEEK_SET);
+			if (ret != 0) {
+				SWBIS_E_FAIL("");
+				return -1;
+			}
+			(*uinformat)->ztypeM = UINFILE_COMPRESSED_NOT;
+			(*uinformat)->typeM = PLAIN_TARBALL_SRC_FILEFORMAT;
+			ret =  uinfile_handle_return(*uinformat,
+				UINFILE_DETECT_FORCEUXFIOFD,
+				UXFIO_BUFTYPE_DYNAMIC_MEM);
+			UINFILE_E_DEBUG2("returning fd=%d", ret);
+			if (path_prefix && strob_strlen(path_prefix) < 4) {
+				strob_close(path_prefix);
+				path_prefix = NULL;
+			}
+			if (path_prefix) {
+				/* fprintf(stderr, "path prefix = [%s]\n", strob_str(path_prefix)); */
+				(*uinformat)->pathname_prefixM = strdup(strob_str(path_prefix));
+				strob_close(path_prefix);
+			} else {
+				fprintf(stderr, "%s: no constant leading pathname prefix in tarball\n", swlib_utilname_get());
+				return -1;
+			}
+			UINFILE_E_DEBUG2("returning ret=%d", ret);
+			strob_close(tmp);
+			return ret;
+		} else if (doieee == 0 && donative == 0 && dodebcontext == 0 &&
+			dodebcontrol == 0 && dodebdata == 0 &&
+			dounrpminstall == 0 &&
+			memcmp(xbuffer, UINFILE_MAGIC_binary_tarball, strlen(UINFILE_MAGIC_binary_tarball)) == 0
+		) {
+			int xeoa;
+			UINFILE_E_DEBUG("type: binary_tarball");
+			UINFILE_E_DEBUG("");
+			ret = taru_tape_buffered_read(refd, xbuffer+peeklen, sizeof(xbuffer) - peeklen);
+			if (ret != (int)sizeof(xbuffer) - (int)peeklen) {
+				return -1;
+			}
+			UINFILE_E_DEBUG2("ret=%d", ret);
+
+			/* This will save the first archive member which is "./", we need the
+			   permissions on this to create in the sw POSIX archive */
+			ret = taru_read_in_tar_header2((*uinformat)->taruM,
+							(*uinformat)->file_hdrM,
+							-1 /* fd */,
+							xbuffer,
+							&xeoa, 0, TARRECORDSIZE);
+			if (ret != TARRECORDSIZE) {
+				return -1;
+			}
+
+			if (
+				memcmp(xbuffer+TARRECORDSIZE, UINFILE_MAGIC_slack "/", strlen(UINFILE_MAGIC_slack "/")) == 0
+			) {
+
+				/* This was too easy,
+				   the "install/" directory was a the front of the package */
+
+				UINFILE_E_DEBUG("");
+				(*uinformat)->ztypeM = UINFILE_COMPRESSED_NOT;
+				(*uinformat)->typeM = SLACK_FILEFORMAT;
+
+				ret =  uinfile_handle_return(*uinformat,
+						UINFILE_DETECT_FORCEUXFIOFD,
+						UXFIO_BUFTYPE_DYNAMIC_MEM);
+				return ret;
+			} else {
+				SWVARFS * swvarfs;
+				struct stat st;
+				char * name;
+				int found;
+
+				UINFILE_E_DEBUG("");
+				/* The only binary tarball recognized is slackware */
+
+				/* This code path handles the ./install directory placed anywhere
+				   in the archive. */
+
+				/* read the entire package into the buffered file descriptor */
+				while ((ret=uxfio_read(refd, xbuffer, TARRECORDSIZE)) > 0); 
+				if (ret < 0) {
+					SWBIS_E_FAIL("read error");
+					return -1;
+				}
+
+				/* now seek back to beginning  */
+
+				ret = uxfio_lseek(refd, 0, SEEK_SET);
+				if (ret != 0) {
+					SWBIS_E_FAIL("");
+					return -1;
+				}
+
+				/* Now read the package and look for the install directory which
+				   may be all the way at the end */
+				swvarfs = swvarfs_opendup(refd, UINFILE_DETECT_NATIVE, (mode_t)0);
+				UINFILE_E_DEBUG("");
+				if (swvarfs == NULL) {
+					SWBIS_E_FAIL("");
+					return -1;
+				}
+				UINFILE_E_DEBUG("");
+				found = 0;
+				while ((name=swvarfs_get_next_dirent(swvarfs, &st)) != NULL && strlen(name)) {
+					/* While we are reading the files, store the leading path prefix
+					   and determine if it is the same for every file. */
+					UINFILE_E_DEBUG2("||||||| [%s]", name);
+					/* Look for the slackware control directory "install/", if we find
+					   one assume it is slackware */
+					if (strcmp(name, UINFILE_MAGIC_slack "/") == 0) {
+						UINFILE_E_DEBUG("found install directory");
+						found = 1;
+						break;
+					}
+				}
+
+				UINFILE_E_DEBUG("");
+
+				ret = uxfio_lseek(refd, 0, SEEK_SET);
+				if (ret != 0) {
+					SWBIS_E_FAIL("");
+					return -1;
+				}
+
+				swvarfs_close(swvarfs);
+				if (found) {
+					(*uinformat)->ztypeM = UINFILE_COMPRESSED_NOT;
+					(*uinformat)->typeM = SLACK_FILEFORMAT;
+					ret =  uinfile_handle_return(*uinformat,
+						UINFILE_DETECT_FORCEUXFIOFD,
+						UXFIO_BUFTYPE_DYNAMIC_MEM);
+				} else {
+					ret = -1;
+				}
+				UINFILE_E_DEBUG2("returning fd=%d", ret);
+				return ret;
+			}
 		} else {
+			struct tar_header *tar_hdr;
 			/*
 			 * If nothing special is detected, assume its tar.
 			 */
+
+			/*
+			if (dounrpminstall) {
+				(*uinformat)->ztypeM = UINFILE_COMPRESSED_RPM;
+			} else {
+			*/
+			UINFILE_E_DEBUG("type: arbitrary tar");
 			if ((*uinformat)->ztypeM == UINFILE_COMPRESSED_NA) {
 				(*uinformat)->ztypeM = UINFILE_COMPRESSED_NOT;
 			}
 			if (taru_tape_buffered_read(refd,
-				buffer + peeklen, 512 - peeklen) != (512 - peeklen)) return -1;
-			if (!strncmp(buffer + 257, "ustar" /*TMAGIC*/, 5)) {
+				xbuffer + peeklen, 512 - peeklen) != (512 - peeklen)) return -1;
+
+			tar_hdr = (struct tar_header *)xbuffer;
+			taru_otoul(tar_hdr->chksum, &tar_hdr_sum);
+
+			if (
+				strncmp(xbuffer + 257, "ustar" /*TMAGIC*/, 5) == 0 ||
+				tar_hdr_sum == taru_tar_checksum(xbuffer)
+			) {
 				/* uncompressed tar */
 				/*
 				 * This could be a deb format package
@@ -609,38 +1058,59 @@ LABEL_REREAD:
 					(*uinformat)->has_leading_slashM =
 						determine_if_has_leading_slash(
 									*uinformat,
-									buffer);
+									xbuffer);
 					ret = uinfile_handle_return(*uinformat,
 							oflags, uxfio_buffer_type);
 					if (ret > 0 && doieee) {
-						ret = uinfile_detect_ieee(*uinformat,
+						int ret2;
+						ret2 = uinfile_detect_ieee(*uinformat,
 										oflags);
+						if (ret2 < 0 && dounrpminstall) {
+							(*uinformat)->ztypeM = UINFILE_COMPRESSED_RPM;
+						} else if (ret2 < 0) {
+							ret = ret2;
+						} else {
+							ret = ret2;
+						}
 					}
 				}
+				/*
 				UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(ret));
+				*/
 				return ret;
 			} else {
-				uxfio_close(refd);
-				swbis_free(*uinformat);
-				fprintf(stderr,
-				"%s: error: uinfile: unrecognized format\n",
-						swlib_utilname_get());
-				return -1;
+				if (dounrpminstall) {
+					/* Try to let swpackage decode it */
+					(*uinformat)->ztypeM = UINFILE_COMPRESSED_RPM;
+				} else {
+					uxfio_close(refd);
+					swbis_free(*uinformat);
+					fprintf(stderr, "%s: error: uinfile: unrecognized tar variant format\n",
+							swlib_utilname_get());
+					return -1;
+				}
 			}
+			/* } */ /* arbitrary tar */
 		}
 	} else {
-		if (!memcmp(buffer, (void *) gzmagic, 2)) {
+		UINFILE_E_DEBUG("");
+		if (!memcmp(xbuffer, (void *) gzmagic, 2)) {
 						/* gzipped compression */
+			UINFILE_E_DEBUG("");
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_GZ;
-		} else if (!memcmp(buffer, (void *) bz2magic, 3)) {
+		} else if (!memcmp(xbuffer, (void *) bz2magic, 3)) {
 							/* bz2 compression */
+			UINFILE_E_DEBUG("");
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_BZ2;
-		} else if (!memcmp(buffer, (void *) Zmagic, 2)) {
+		} else if (!memcmp(xbuffer, (void *) Zmagic, 2)) {
 							/* Unix compress */
+			UINFILE_E_DEBUG("");
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_Z;
 		} else {
+			UINFILE_E_DEBUG("");
 			ret =  uinfile_handle_return(*uinformat,
 						oflags, uxfio_buffer_type);
+			UINFILE_E_DEBUG2("ret=%d", ret);
 			return ret;
 		}
 	}
@@ -650,7 +1120,7 @@ LABEL_REREAD:
 	/*     -or- its an RPM and UINFILE_DETECT_RPM is ON.		 */
 	/* ------------------------------------------------------------- */
 
-	UINFILE_E_DEBUG("HERE its compressed");
+	UINFILE_E_DEBUG("HERE its compressed !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
 	{ /* Compressed file. */
 
 	SHCMD *unzip[2];
@@ -659,28 +1129,27 @@ LABEL_REREAD:
 	unzip[1] = (SHCMD*)NULL;
 
 	UINFILE_E_DEBUG2("HERE current_pos_=%d", (int)((*uinformat)->current_pos_));
-	if (uxfio_lseek(refd, (off_t)((*uinformat)->current_pos_), SEEK_SET) !=
-		(off_t)((*uinformat)->current_pos_)
-	) {
-		fprintf(stderr, "%s: %s: uxfio_lseek error. 001.1 pos=%d\n",
-			swlib_utilname_get(), __FILE__, ((*uinformat)->current_pos_)); 
-		return -1;
+	if (!dodebdata) {
+		if (uxfio_lseek(refd, (off_t)((*uinformat)->current_pos_), SEEK_SET) !=
+			(off_t)((*uinformat)->current_pos_)
+		) {
+			fprintf(stderr, "%s: %s: uxfio_lseek error. 001.1 pos=%d\n",
+				swlib_utilname_get(), __FILE__, ((*uinformat)->current_pos_)); 
+			return -1;
+		}
 	}
 	if (zcat_command && (*uinformat)->ztypeM == UINFILE_COMPRESSED_Z) {
 		unzip[0] = shcmd_open();
 		shcmd_add_arg(unzip[0], "compress");
 		shcmd_add_arg(unzip[0], "-cd");
 	} else if ((*uinformat)->ztypeM == UINFILE_COMPRESSED_DEB) {
-		/*
-		 * Make some tests on the first few bytes of the package
-		 */
 		UINFILE_E_DEBUG("in deb code");
-		if (memcmp(((char*)(buffer)) + UINFILE_DEB_CONTROL_OFFSET,
+		if (memcmp(((char*)(xbuffer)) + deb_gz_offset,
 				gzmagic, 3) == 0) {
 			unzip[0] = shcmd_open();
 			shcmd_add_arg(unzip[0], "gzip");
 			shcmd_add_arg(unzip[0], "-d");
-		} else if (memcmp(((char*)(buffer)) + UINFILE_DEB_CONTROL_OFFSET,
+		} else if (memcmp(((char*)(xbuffer)) + deb_gz_offset,
 				bz2magic, 3) == 0) {
 			/*
 			 * preemptive support for bzip2'ed debian packages
@@ -692,36 +1161,15 @@ LABEL_REREAD:
 			/*
 			 * error, the compressed magic not found at the expected offset
 			 */
+			fprintf(stderr, "%s: internal error %s: at line %d\n", swlib_utilname_get(), __FILE__, __LINE__);
 			return -1;
 		} 
+
 		/*
-		 * grab the size of the control.tar.gz archive from member
+		 * The file is now positioned at the control.tar.gz file
 		 */
-		s_size = (char*)buffer + 
-				( strlen(UINFILE_MAGIC_ar) +
-				TARU_AR_HEADER_SIZE +
-				4 + /* member payload length i.e. "2.0\n" */
-				TARU_AR_SIZE_OFFSET );
-
-		control_gz_size = swlib_atoi(s_size, NULL);
-
-		if (control_gz_size <= 0 || control_gz_size > 100000000) {
-			/*
-			 * sanity fails
-			 */
-			return -1;
-		}
-
-		if (uxfio_lseek(refd, UINFILE_DEB_CONTROL_OFFSET, SEEK_SET) != UINFILE_DEB_CONTROL_OFFSET) {
-			fprintf(stderr, "%s: %s: uxfio_lseek error. 001.3 pos=%d\n",
-				swlib_utilname_get(), __FILE__, ((*uinformat)->current_pos_)); 
-			return -1;
-		} else {
-			/*
-			 * The file is now positioned at the control.tar.gz file
-			 */
-			; /* continue */
-		}
+		UINFILE_E_DEBUG("leaving deb code");
+		; /* continue */
 	} else if ((*uinformat)->ztypeM == UINFILE_COMPRESSED_GZ) {
 		UINFILE_E_DEBUG("in gzip code");
 		unzip[0] = shcmd_open();
@@ -743,7 +1191,7 @@ LABEL_REREAD:
 		unzip[0] = shcmd_open();
 		/* shcmd_add_arg(unzip[0], "swbis"); */
 		shcmd_add_arg(unzip[0], SWBISBINDIR "/swpackage");
-		shcmd_add_arg(unzip[0], "--unrpm");
+		shcmd_add_arg(unzip[0], "--to-swbis");
 		/*
 		shcmd_add_arg(unzip[0], "--no-defaults");
 		shcmd_add_arg(unzip[0], "-W");
@@ -786,12 +1234,16 @@ LABEL_REREAD:
 		}
 		uinfile_add_pid(*uinformat, (int)pid);
 		close(pipe_fd[1]);
-		refd=uxfio_opendup(pipe_fd[0], uxfio_buffer_type);
+		refd = uxfio_opendup(pipe_fd[0], uxfio_buffer_type);
+		/*
 		UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(refd));
-
+		*/
 		/*
 		 * Re-read the alledgedly uncompressed file.
 		 */
+		if (dodebdata) return refd;
+		dounrpminstall = 0;
+		UINFILE_E_DEBUG("GOTO'ING !!!!!!!!!!!!!!!!!!!!!!!!!! ");
 		goto LABEL_REREAD;
 	} else {
 		/* 
@@ -834,7 +1286,7 @@ LABEL_REREAD:
 					 * pump the exact size of the control.tar.gz file
 					 */
 					UINFILE_E_DEBUG("HERE");
-					pump_amount = control_gz_size;
+					pump_amount = (intmax_t)deb_gz_size;
 				} else {
 					/*
 					 * pump until the end
@@ -855,16 +1307,40 @@ LABEL_REREAD:
 				} else if (a_pid == 0) {
 					UINFILE_E_DEBUG("HERE ");
 					close(zpipe[0]);
-					if ((*uinformat)->ztypeM != UINFILE_COMPRESSED_DEB) {
-						uxfio_fcntl(refd,
-						UXFIO_F_ARM_AUTO_DISABLE, 1);
+					if (
+						(1 || (*uinformat)->ztypeM != UINFILE_COMPRESSED_DEB) &&
+						dodebdata == 0
+					) {
+						UINFILE_E_DEBUG("HERE at UXFIO_F_ARM_AUTO_DISABLE");
+						uxfio_fcntl(refd, UXFIO_F_ARM_AUTO_DISABLE, 1);
 						UINFILE_E_DEBUG2("HERE current_pos_=%d", (int)((*uinformat)->current_pos_));
 						if (uxfio_lseek(refd, (off_t)((*uinformat)->current_pos_), SEEK_SET) < 0)
 							fprintf(stderr,
 						"uxfio_lseek error in child 002.\n");
 					}
-					taru_pump_amount2(zpipe[1],
-								refd, pump_amount, -1); 
+
+					if (
+						dodebdata &&
+						(*uinformat)->n_deb_peeked_bytesM > 0
+					) {
+						/* Here we are opening the data.tar.gz file and we must
+						   write the peek'ed bytes to reconstitute the stream */
+						if (
+							uxfio_write(zpipe[1],
+								(void*)
+								((unsigned char*)((*uinformat)->deb_peeked_bytesM) +
+										deb_gz_offset - SARHDR
+								)
+								,
+								(*uinformat)->n_deb_peeked_bytesM) !=
+								(*uinformat)->n_deb_peeked_bytesM
+						   ) {
+							fprintf(stderr, "%s: ar archive error %s: at line %d\n",
+								swlib_utilname_get(), __FILE__, __LINE__);
+							_exit(1);
+						    }
+					}
+					taru_pump_amount2(zpipe[1], refd, pump_amount - ((*uinformat)->n_deb_peeked_bytesM), -1); 
 					close(zpipe[1]);
 					uxfio_close(refd);
 					_exit(0);
@@ -1111,6 +1587,8 @@ uinfile_close(UINFORMAT * uinformat)
 	if (uinformat->verboseM)	
 		fprintf(stderr,
 			"Leaving uinfile_close with status %d\n", ret);
+	if (uinformat->taruM)
+		taru_delete(uinformat->taruM);
 	swbis_free(uinformat);
 	return ret;
 }
