@@ -4,14 +4,15 @@
 #ifndef uxfio_20020419_h
 #define uxfio_20020419_h
 
+#include "swuser_config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include "swuser_config.h"
 
 /* uxfio ---  Unix  eXtended  File  Input/Output  API */
 
@@ -64,6 +65,7 @@
 					  	memory then exit. */
 #define UXFIO_F_SET_OUTPUT_BLOCK_SIZE -11743  /* Set output block size,
 						Zero (0) for no buffering. */
+#define UXFIO_F_GET_VEOF 	-11744 
 
 /* I/O Controls  Hopefully none of these match a real uxix system. */
 #define UXFIO_IOCTL_SET_STATBUF 		-12725
@@ -79,6 +81,9 @@
 /*-------------------------------------------------------------*/
 /*------- UXFIO API (PUBLIC FUNCTIONS)------------------------*/
 
+
+int uxfio_uxfio_get_nopen(void);
+
 int uxfio_open ( char * path, int oflag, mode_t mode );
 
 int uxfio_opendup ( int unix_fd, int BUF_TYPE );
@@ -88,9 +93,11 @@ int uxfio_close ( int uxfio_fides );
 
 ssize_t uxfio_read ( int uxfio_fildes, void *buf, size_t nbyte );
 
+ssize_t uxfio_sfa_read(int uxfio_fildes, void *buf, size_t nbyte ); /* actually a polling read */
+
 ssize_t uxfio_sfread(int uxfio_fildes, void *buf, size_t nbyte ); /* actually a polling read */
 
-ssize_t uxfio_unix_safe_read(int unix_fd, void * buf, int nbyte);
+ssize_t uxfio_unix_safe_atomic_read(int unix_fd, void * buf, size_t nbyte);
 
 /* ssize_t uxfio_polling_read(int unix_fd, void * buf, int nbyte); */
 
@@ -131,10 +138,10 @@ char *  uxfio_dump_string_s(int uxfio_fildes, char * prefix);
 int uxfio_fileinsert(int dst_fd, int src_fd, int count);
 void * uxfio_get_object_address(int uxfiofd);
 ssize_t uxfio_full_write (int desc, const char *ptr, size_t len);
-ssize_t uxfio_atomic_read(int fd, void * buf, size_t nbyte);
-ssize_t uxfio_atomic_write(int fd, void * buf, size_t nbyte);
-ssize_t uxfio_intr_read(int fd, void * buf, size_t nbyte);
-ssize_t uxfio_intr_write(int fd, void * buf, int nbyte);
+ssize_t uxfio_unix_atomic_read(int fd, void * buf, size_t nbyte);
+ssize_t uxfio_unix_atomic_write(int fd, void * buf, size_t nbyte);
+ssize_t uxfio_unix_safe_read(int fd, void * buf, size_t nbyte);
+ssize_t uxfio_unix_safe_write(int fd, void * buf, int nbyte);
 char * uxfio_get_fd_mem(int uxfio_fildes, int * data_len);
 
 #endif
